using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.XR.ARFoundation;

namespace SpellBoundAR.ARImageTracking
{
    public class MultipleImageTracking : MonoBehaviour
    {
        public static event Action<ARTrackedImage, ImageTargetData> OnAnyTargetTracked;
        public static event Action<ARTrackedImage, ImageTargetData> OnAnyTargetRemoved;

        [Header("Settings")]
        public bool instantiateContent = true;

        [Header("References")]
        [SerializeField] private ARTrackedImageManager trackedImageManager;

        [Header("Image Target Data")]
        [SerializeField] private List<ImageTargetData> imageTargets;

        [Header("Map target data to their content")]
        private GameObject _activeContent = null;
        private Dictionary<ImageTargetData, GameObject> _content;

        private void Awake()
        {
            _content = new Dictionary<ImageTargetData, GameObject>();
            foreach (ImageTargetData targetData in imageTargets) _content.Add(targetData, null);
        }
    
        private void OnEnable() 
        {
            if (trackedImageManager) trackedImageManager.trackedImagesChanged += ImageStatusChanged;
        }

        private void OnDisable()
        {
            if (trackedImageManager) trackedImageManager.trackedImagesChanged -= ImageStatusChanged;
        }

        private ImageTargetData GetTargetData(ARTrackedImage image) 
        {
            return imageTargets.Find(item => item.ReferenceImage.name == image.referenceImage.name);
        }

        private void ImageStatusChanged(ARTrackedImagesChangedEventArgs status)
        {
            foreach (ARTrackedImage image in status.added) AddContent(image);
            foreach (ARTrackedImage image in status.updated) UpdateContent(image);
            foreach (ARTrackedImage image in status.removed) RemoveContentByImage(image);
        }

        private void AddContent(ARTrackedImage image)
        {
            ImageTargetData imageTargetData = GetTargetData(image);
            if (imageTargetData == null) return;
            if (instantiateContent)
            {
                if (!_content.ContainsKey(imageTargetData) || _content[imageTargetData] != null || _activeContent != null) return;
                GameObject content = imageTargetData.Content;
                if (!content) return;
                _content[imageTargetData] = Instantiate(content);
                _activeContent = _content[imageTargetData];
                Transform imageTransform = image.transform;
                _content[imageTargetData].transform.position = imageTransform.position;
                _content[imageTargetData].transform.rotation = imageTransform.rotation * imageTargetData.RotationOffset;
            }
        
            imageTargetData.OnTracked();
            OnAnyTargetTracked?.Invoke(image, imageTargetData);
        }

        private void UpdateContent(ARTrackedImage image) 
        {
            ImageTargetData imageTargetData = GetTargetData(image);
            if (imageTargetData == null) return;
            switch (image.trackingState) 
            {
                case UnityEngine.XR.ARSubsystems.TrackingState.Tracking:
                    if (_content.ContainsKey(imageTargetData) && _content[imageTargetData] == null) AddContent(image);
                    if (_content.ContainsKey(imageTargetData) && _content[imageTargetData] != null) 
                    {
                        Transform imageTransform = image.transform;
                        _content[imageTargetData].transform.position = imageTransform.position;
                        _content[imageTargetData].transform.rotation = imageTransform.rotation * imageTargetData.RotationOffset;
                    }
                    break;
                case UnityEngine.XR.ARSubsystems.TrackingState.Limited:
                case UnityEngine.XR.ARSubsystems.TrackingState.None:
                    if (_content.ContainsKey(imageTargetData) && _content[imageTargetData] != null) RemoveContent(image, imageTargetData);
                    break;
            }
        }

        private void RemoveContentByImage(ARTrackedImage image) 
        {
            RemoveContent(image, GetTargetData(image));
        }

        private void RemoveContent(ARTrackedImage image, ImageTargetData imageTargetData) 
        {
            if (imageTargetData == null)  return;
            if (instantiateContent)
            {
                if (!_content.ContainsKey(imageTargetData) || _content[imageTargetData] == null) return;
                Destroy(_content[imageTargetData].gameObject);
                _content[imageTargetData] = null;
                _activeContent = null;
            }
        
            Resources.UnloadUnusedAssets();
            OnAnyTargetRemoved?.Invoke(image, imageTargetData);
        }
    }
}
